/*============================================================================
USP_ReadErrorLog.sql
Written by Taiob M Ali
SqlWorldWide.com


This script will search all available SQL Server Error Log or SQL Server Agent Error Log
for a string.  You can narrow the result set by seraching for a second string with in the first result set.

In writing this script I took help from below site to find number of logs available to search.
https://ask.sqlservercentral.com/questions/99484/number-of-error-log-files.html
@logFileType= 1 for SQL Server Error Log, 2 for SQL Server Agent Error Log
@searchString1 =any string to search for
@searchString2 =filter the search result further
@start =search start from, if NULL all files will be searched.
@end   =search end at,   if NULL all files will be searched. 
@sortOrder = by default ascending, use 'desc' if you need the result in descending order.


Instruction to run this script
--------------------------------------------------------------------------
Sample call 
Searching in SQL Server Error Log for Shutdown and then filter by initiated. Looking inside a time range.

EXEC USP_ReadErrorLog 
	@logFileType   = 1, 
	@searchString1 = N'Shutdown',
	@searchString2 = N'initiated',
	@start         = '2017-08-05 09:37',
	@end           = '2017-08-07 10:37',   
	@sortOrder     = N'desc'
									 
Searching in SQL Server Agent Error Log for cpu and then filter by condition
EXEC USP_ReadErrorLog 
	@logFileType   = 2, 
	@searchString1 = N'cpu',
	@searchString2 = N'condition',
	@start         = NULL ,
	@end           = NULL,   
	@sortOrder     = N'desc'


============================================================================*/
USE [DBAdatabase]
GO

IF NOT EXISTS (SELECT * FROM sys.objects WHERE type = 'P' AND OBJECT_ID = OBJECT_ID('dbo.USP_ReadErrorLog'))
   EXEC('CREATE PROCEDURE [dbo].[USP_ReadErrorLog] AS BEGIN SET NOCOUNT ON; END')
GO
ALTER PROCEDURE dbo.USP_ReadErrorLog
 @logFileType   smallint      = 1,       --Default to SQL Server Error Log, 2 =for SQL Server Agent Error Log 
 @searchString1 nvarchar(256) = NULL,    --NULL will output everything
 @searchString2 nvarchar(256) = NULL ,   --NULL 
 @start         datetime      = NULL ,   --'2017-08-05 09:37'  --Start of search, if NULL start from available log file
 @end           datetime      = NULL,    --'2017-08-05 10:37'    --End of search, if NULL till the end of available log file
 @sortOrder     nvarchar (4)  = N'desc' --or N'asc'
AS
BEGIN
DECLARE @logno int = 0
DECLARE @ErrorLog nvarchar(4000)
DECLARE @ErrorLogPath nvarchar(4000)
DECLARE @NumberOfLogfiles int

DECLARE @FileList AS TABLE (
  subdirectory nvarchar(4000) NOT NULL 
  ,DEPTH bigint NOT NULL
  ,[FILE] bigint NOT NULL
 );
 
IF EXISTS (SELECT *
FROM	dbo.sysobjects
WHERE   id = OBJECT_ID(N'[dbo].[#errorlog]')
)
DROP TABLE [dbo].[#errorlog]

CREATE TABLE #ErrorLog (LogDate datetime, PrcessInfo varchar(20), [Text] nvarchar(4000))

SELECT @ErrorLog     = CAST(SERVERPROPERTY(N'errorlogfilename') AS NVARCHAR(4000));
SELECT @ErrorLogPath = SUBSTRING(@ErrorLog, 1, LEN(@ErrorLog) - CHARINDEX(N'\', REVERSE(@ErrorLog))) + N'\';
 
 INSERT INTO @FileList
 EXEC xp_dirtree @ErrorLogPath, 0, 1;
 
--Reading how many files available

IF(@logFileType=1)--SQL Server Error Log
	BEGIN
		SET @NumberOfLogfiles = (SELECT COUNT(*) FROM @FileList WHERE [@FileList].subdirectory LIKE N'ERRORLOG%');
	END
ELSE             --SQL Server Agent Error Log
	BEGIN
		SET @NumberOfLogfiles = (SELECT COUNT(*) FROM @FileList WHERE [@FileList].subdirectory LIKE N'SQLAGENT%');
	END

--Iterate through each log files
WHILE (@logno<@NumberOflogfiles )
	BEGIN
			INSERT INTO #ErrorLog
			EXEC master.dbo.xp_readerrorlog @logno, @logFileType, @searchString1, @searchString2, @start, @end, NULL ;
			SET @logno = @logno+1
	END

-- Reading the data
SELECT * 
	FROM #ErrorLog
ORDER BY 
CASE WHEN @sortOrder ='desc'
	THEN logdate END desc,
CASE WHEN @sortOrder ='asc'
THEN logdate END 
END